#!/usr/local/bin/guile -s
!#
;;;; Web Gallery Display Engine
;;;; Copyright (C) 2004, 2005 Sergey Poznyakoff
;;;; 
;;;; This program is free software; you can redistribute it and/or modify
;;;; it under the terms of the GNU General Public License as published by
;;;; the Free Software Foundation; either version 2 of the License, or
;;;; (at your option) any later version.
;;;;
;;;; This program is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;; GNU General Public License for more details.
;;;;
;;;; You should have received a copy of the GNU General Public License
;;;; along with this program; if not, write to the Free Software
;;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;;;;

(use-modules (www cgi))
(use-modules (ice-9 regex))
(cgi:init)

(define cgi-url "cgi-bin/webgal.cgi")
(define gallery-template-file "/home/gray/public_html/gallery/@@lang@@/gallery.html")
(define image-template-file "/home/gray/public_html/gallery/@@lang@@/image.html")
(define index-template-file "/home/gray/public_html/gallery/@@lang@@/idx.html")
(define title-template-file "@@lang@@/TITLE")
(define gallery-index-url-prefix "preview")
(define gallery-index-dir "preview") 
(define gallery-base "gallery")
(define gallery-base-dir "/home/gray/public_html/gallery")
(define image-suffix "jpg")
(define image-rx (string-append "(.*)\." image-suffix))
(define text-template "@@lang@@/@@image-id@@.txt")
(define max-columns 6)

(define (gallery-id) 
  (cgi:value "gallery"))

(define (image-id)
  (cgi:value "image"))

(define (language)
  (or (cgi:value "lang") "pl"))

(define (make-index-dir-name)
  (let ((gallery (gallery-id)))
    (if gallery
	(string-append gallery-base-dir "/" gallery "/" gallery-index-dir)
	(throw 'no-gallery-id))))

(define (scan-index-dir dir-name)
;  (display "SCANNING ")(display dir-name)(display "<br>")
  (let ((d (opendir dir-name)))
    (let ((result (do ((dir (readdir d) (readdir d))
		       (img-list '()))
		      ((eof-object? dir)
		       (stable-sort img-list string<?))
		    (cond
		     ((string-match image-rx dir) =>
		      (lambda (match)
			(set! img-list (cons (match:substring match 1)
					     img-list))))))))
      (closedir d)
;      (display "RESULT ")(display result)(display "<br>")
      result)))

(define (read-short name . rest)
  (let ((print? (or (null? rest) (car rest)))
	(found? #f))
    (call-with-current-continuation
     (lambda (return)
       (do ((line (read-line) (read-line)))
	   ((eof-object? line) (cons found? line))
	 (cond
	  ((string-match "short: *" line) =>
	   (lambda (match)
	     (cond
	      (print?
	       (set! found? #t)
	       (display " ")
	       (display (match:suffix match))))))
	  (else
	   (return (cons found? line)))))))))

(define (read-file name . rest)
  (catch #t
	 (lambda ()
	   (with-input-from-file name
	     (lambda ()
	       (letrec ((display-rest (lambda (x)
					(do ((line x (read-line)))
					    ((eof-object? line) #f)
					  (display " ")
					  (display line)))))
		 
		 (if (and (not (null? rest)) (car rest))
		     (let ((res (read-short name)))
		       (if (not (car res))
			   (display-rest (cdr res))))
		     (let ((res (read-short name #f)))
		       (display-rest (cdr res))))))))
	 (lambda args
	   #f)))

(define (find-image-neighbors name)
  (call-with-current-continuation
   (lambda (return)
     (do ((flist (scan-index-dir (make-index-dir-name)) (cdr flist))
	  (prev #f))
	 ((null? flist) (cons prev #f))
       (let ((item (car flist)))
	 (if (string=? item name)
	     (return (cons prev (and (not (null? (cdr flist)))
				     (cadr flist))))
	     (set! prev item)))))))

(define image-neighbors #f)

(define (current-image-neighbors)
  (if (not image-neighbors)
      (set! image-neighbors (find-image-neighbors (image-id))))
  image-neighbors)

(define (main-form)
  (display "<FORM ACTION=\"")
  (display cgi-url)
  (display "\" METHOD=POST>")
  
  (let ((filelist (scan-index-dir (make-index-dir-name))))
    (display "<div align=\"center\" class=\"gallery\"><table class=\"gallery\" rules=\"groups\">")
    (do ((row 0 (1+ row)))
	((null? filelist) #f)
      (display "<tbody>\n")
      (display "<tr class=\"image\">")
      
      (do ((col 0 (1+ col))
	   (flist filelist (cdr flist)))
	  ((or (null? flist) (= col max-columns)) #f)
	(display "<td class=\"image\" align=\"center\"><a href=\"")

        (display (image-url (gallery-id) (car flist)))

	(display "\"><img border=\"0\" src=\"")
	(display (string-append
		  gallery-base "/"
		  (gallery-id) "/"
		  gallery-index-url-prefix "/" (string-append
						(car flist) "." image-suffix)))
	(display "\"></img></a></td>"))
      
      (display "</tr>\n<tr class=\"comment\">")

      (do ((col 0 (1+ col)))
	  ((or (null? filelist) (= col max-columns)) #f)
	(display "<td class=\"comment\">")
	(read-file (string-append
		    gallery-base-dir
		    "/"
		    (gallery-id)
		    "/"
		    gallery-index-dir
		    "/" (expand-var "@@image-id@@" (car filelist)
				    (make-file-name text-template)))
		   #t)
	(display "</td>")
	(set! filelist (cdr filelist)))

      (display "</tr>")
      (newline)
      (display "</tbody>")
      (newline))
    (display "</table></div>")))

(define (image-form image)
  (display "<div align=\"center\">")
  (display "<img src=\"")
  (display (string-append gallery-base "/" (gallery-id) "/" image))
  (display "\"></img></div>"))

(define (image-url id img)
  (apply string-append 
          (append
           (list
            cgi-url
	    "?lang=" (language)
	    "&amp;"
            "gallery=" id
            "&amp;"
            "image=" img))))

(define explist
  (list (cons "@@program-name@@"  (lambda (id)
				    (display cgi-url)))
	(cons "@@script@@"        (lambda (id)
				    (display cgi-url)
				    (display "?lang=")
				    (display (language))))
	(cons "@@gallery-id@@"    (lambda (id)
				    (display (gallery-id))))
	(cons "@@image-id@@"      (lambda (id)
				    (display (image-id))))
	(cons "@@gallery-url@@"   (lambda (id)
				    (display cgi-url)
				    (display "?lang=")
				    (display (language))
				    (display "&gallery=")
				    (display id)))
	(cons "@@gallery-descr@@"  (lambda (id)
				     (let ((name (string-append
						  gallery-base-dir "/" id "/" 
						  gallery-index-dir "/"
						  (make-file-name title-template-file))))
				       (if (access? name R_OK)
					   (read-file name)))))))
			    
(define (expand-template template dir)
  (let loop ((template template))
    (cond
     ((string-index template #\@) =>
      (lambda (w)
	(display (substring template 0 w))
	(if (and (< (+ w 2) (string-length template))
		 (char=? (string-ref template (1+ w)) #\@))
	    (let ((end-pos (string-index template #\@ (+ w 2))))
	      (if (and end-pos
		       (< (1+ end-pos) (string-length template))
		       (char=? (string-ref template (1+ end-pos)) #\@))
		  (let* ((name (substring template w (+ end-pos 2)))
			 (entry (assoc name explist)))
		    (cond
		     (entry
		      ((cdr entry) dir)
		      (loop (substring template (+ end-pos 2))))
		     (else
		      (display "@@")
		      (loop (substring template (+ w 2))))))
		  (begin
		    (display "@")
		    (loop (substring template (+ w 1))))))
	    (begin
	      (display "@")
	      (loop (substring template (1+ w)))))))
     (else
      (display template)))))
	     
	
(define (make-gallery-index template)
  (let ((dir (opendir gallery-base-dir)))
    (do ((d (readdir dir) (readdir dir)))
	((eof-object? d) #f)
      (if (not (or (string=? d ".") (string=? d "..")))
	  (catch #t
		 (lambda ()
		   (let ((st (stat (string-append gallery-base-dir "/" d))))
		     (case (stat:type st)
		       ((directory)
			(expand-template template d)))))
		 (lambda args
		   #f))))))
		     
(define (make-html)
  (let ((accumulator #f)
        (mode #f))
    (do ((line (read-line) (read-line)))
	((eof-object? line) #f)
      (cond
       (accumulator
	(cond
	 ((string=? line "@@end@@")
	  (make-gallery-index accumulator)
          (set! accumulator #f))
         ((and (>= (string-length line) 2)
               (string=? (substring line 0 2) "?>")) 
          (eval-string accumulator)
	  (set! accumulator #f))
	 (else
	  (set! accumulator (string-append accumulator "\n" line)))))
       ((string=? line "@@title@@")
	(let ((name (string-append
		     gallery-base-dir "/" (gallery-id) "/" 
		     gallery-index-dir "/" (make-file-name title-template-file))))
	  (if (access? name R_OK)
	      (read-file name))))
       ((string=? line "@@gallery@@")
	(main-form))
       ((and (>= (string-length line) 7)
             (string=? (substring line 0 7) "<?guile"))
        (set! accumulator (substring line 7)))
       ((string=? line "@@image@@")
	(image-form (image-id)))
       ((string=? line "@@text@@")
	(read-file (string-append
		    gallery-base-dir "/" (gallery-id) "/" 
		    gallery-index-dir "/" (expand-var "@@image-id@@" (image-id)
						      (make-file-name text-template)))))
       ((string=? line "@@repeat@@")
	(set! accumulator ""))
       (else
	(expand-template line (gallery-id)) 
	(newline))))))  

(define (expand-var var value tmpl)
  (cond
   ((string-match (string-append "(.*)" var "(.*)") tmpl) =>
    (lambda (match)
      (make-file-name (string-append (match:substring match 1)
				     value
				     (match:substring match 2)))))
   (else
    tmpl)))

(define (make-file-name tmpl)
  (expand-var "@@lang@@" (language) tmpl))

;;; Main
(display "Content-type: text/html; charset=utf-8\r\n\r\n")

(catch #t
       (lambda ()
	 (let ((image (image-id))
	       (gallery (gallery-id)))
	   (with-input-from-file
	       (make-file-name 
		(cond
		 (image image-template-file)
		 (gallery gallery-template-file)
		 (else index-template-file)))
	     make-html))) 
       (lambda args
	 (display "Exception caught: ")
	 (display args)))


;;;; Local variables:
;;;; mode: Scheme
;;;; buffer-file-coding-system: utf-8
;;;; End:
